@@ -20,7 +20,7 @@ from __future__ import annotations
 import collections.abc
 import functools
 from collections import Counter
-from collections.abc import Iterator, KeysView, Mapping
+from collections.abc import Iterator, KeysView, Mapping, Sequence
 from typing import TYPE_CHECKING, NamedTuple
 
 from sqlalchemy import and_, func, or_, select
@@ -31,15 +31,16 @@ from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
 from airflow.utils.state import TaskInstanceState
 
 if TYPE_CHECKING:
+    from sqlalchemy.engine import Row
     from sqlalchemy.orm import Session
-    from sqlalchemy.sql.expression import ColumnOperators
+    from sqlalchemy.sql import ColumnElement
 
-    from airflow.models.mappedoperator import MappedOperator
     from airflow.models.taskinstance import TaskInstance
+    from airflow.serialization.definitions.mappedoperator import Operator
     from airflow.serialization.definitions.taskgroup import SerializedMappedTaskGroup
-    from airflow.serialization.serialized_objects import SerializedBaseOperator
     from airflow.ti_deps.dep_context import DepContext
     from airflow.ti_deps.deps.base_ti_dep import TIDepStatus
+    from airflow.typing_compat import Unpack
 
 
 class _UpstreamTIStates(NamedTuple):
@@ -68,11 +69,12 @@ class _UpstreamTIStates(NamedTuple):
 
         :param finished_upstreams: all the finished upstreams of the dag_run
         """
-        counter: dict[str, int] = Counter()
-        setup_counter: dict[str, int] = Counter()
+        counter: Counter[str] = Counter()
+        setup_counter: Counter[str] = Counter()
         for ti in finished_upstreams:
             if TYPE_CHECKING:
                 assert ti.task
+                assert ti.state
             curr_state = {ti.state: 1}
             counter.update(curr_state)
             if ti.task.is_setup:
@@ -130,8 +132,8 @@ class TriggerRuleDep(BaseTIDep):
         """
         from airflow.exceptions import NotMapped
         from airflow.models.expandinput import NotFullyPopulated
-        from airflow.models.mappedoperator import is_mapped
         from airflow.models.taskinstance import TaskInstance
+        from airflow.serialization.definitions.mappedoperator import is_mapped
 
         task = ti.task
         if TYPE_CHECKING:
@@ -145,7 +147,7 @@ class TriggerRuleDep(BaseTIDep):
             This extra closure allows us to query the database only when needed,
             and at most once.
             """
-            from airflow.models.mappedoperator import get_mapped_ti_count
+            from airflow.serialization.definitions.mappedoperator import get_mapped_ti_count
 
             return get_mapped_ti_count(task, ti.run_id, session=session)
 
@@ -222,7 +224,7 @@ class TriggerRuleDep(BaseTIDep):
                 return True
             return False
 
-        def _iter_upstream_conditions(relevant_tasks: dict) -> Iterator[ColumnOperators]:
+        def _iter_upstream_conditions(relevant_tasks: dict) -> Iterator[ColumnElement]:
             # Optimization: If the current task is not in a mapped task group,
             # it depends on all upstream task instances.
             from airflow.models.taskinstance import TaskInstance
@@ -254,7 +256,7 @@ class TriggerRuleDep(BaseTIDep):
                     yield and_(TaskInstance.task_id == upstream_id, TaskInstance.map_index == map_indexes)
 
         def _evaluate_setup_constraint(
-            *, relevant_setups: Mapping[str, SerializedBaseOperator | MappedOperator]
+            *, relevant_setups: Mapping[str, Operator]
         ) -> Iterator[tuple[TIDepStatus, bool]]:
             """
             Evaluate whether ``ti``'s trigger rule was met as part of the setup constraint.
@@ -370,7 +372,8 @@ class TriggerRuleDep(BaseTIDep):
                 upstream = len(upstream_tasks)
                 upstream_setup = sum(1 for x in upstream_tasks.values() if x.is_setup)
             else:
-                task_id_counts = session.execute(
+                # The below type annotation is acceptable on SQLA2.1, but not on 2.0
+                task_id_counts: Sequence[Row[Unpack[tuple[str, int]]]] = session.execute(  # type: ignore[type-arg]
                     select(TaskInstance.task_id, func.count(TaskInstance.task_id))
                     .where(TaskInstance.dag_id == ti.dag_id, TaskInstance.run_id == ti.run_id)
                     .where(or_(*_iter_upstream_conditions(relevant_tasks=upstream_tasks)))
@@ -618,13 +621,7 @@ class TriggerRuleDep(BaseTIDep):
 
         if not task.is_teardown:
             # a teardown cannot have any indirect setups
-            relevant_setups: dict[str, MappedOperator | SerializedBaseOperator] = {
-                # TODO (GH-52141): This should return scheduler types, but
-                # currently we reuse logic in SDK DAGNode.
-                t.task_id: t  # type: ignore[misc]
-                for t in task.get_upstreams_only_setups()
-            }
-            if relevant_setups:
+            if relevant_setups := {t.task_id: t for t in task.get_upstreams_only_setups()}:
                 for status, changed in _evaluate_setup_constraint(relevant_setups=relevant_setups):
                     yield status
                     if not status.passed and changed:
