@@ -16,7 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-import { Box, HStack, Text, IconButton, Button, ButtonGroup } from "@chakra-ui/react";
+import { Box, HStack, IconButton, Text } from "@chakra-ui/react";
 import { keyframes } from "@emotion/react";
 import dayjs from "dayjs";
 import { useState } from "react";
@@ -27,6 +27,7 @@ import { useLocalStorage } from "usehooks-ts";
 
 import { useCalendarServiceGetCalendar } from "openapi/queries";
 import { ErrorAlert } from "src/components/ErrorAlert";
+import { ButtonGroupToggle } from "src/components/ui/ButtonGroupToggle";
 
 import { CalendarLegend } from "./CalendarLegend";
 import { DailyCalendarView } from "./DailyCalendarView";
@@ -88,9 +89,9 @@ export const Calendar = () => {
   }
 
   return (
-    <Box p={6}>
+    <Box data-testid="dag-calendar-root" p={6}>
       <ErrorAlert error={error} />
-      <HStack justify="space-between" mb={6}>
+      <HStack data-testid="calendar-header" justify="space-between" mb={6}>
         <HStack gap={4} mb={4}>
           {granularity === "daily" ? (
             <HStack gap={2}>
@@ -106,6 +107,7 @@ export const Calendar = () => {
                 _hover={selectedDate.year() === currentDate.year() ? {} : { textDecoration: "underline" }}
                 color={selectedDate.year() === currentDate.year() ? "fg.info" : "inherit"}
                 cursor={selectedDate.year() === currentDate.year() ? "default" : "pointer"}
+                data-testid="calendar-current-period"
                 fontSize="xl"
                 fontWeight="bold"
                 minWidth="120px"
@@ -153,6 +155,7 @@ export const Calendar = () => {
                     ? "default"
                     : "pointer"
                 }
+                data-testid="calendar-current-period"
                 fontSize="xl"
                 fontWeight="bold"
                 minWidth="120px"
@@ -178,39 +181,23 @@ export const Calendar = () => {
             </HStack>
           )}
 
-          <ButtonGroup attached size="sm" variant="outline">
-            <Button
-              colorPalette="brand"
-              onClick={() => setGranularity("daily")}
-              variant={granularity === "daily" ? "solid" : "outline"}
-            >
-              {translate("calendar.daily")}
-            </Button>
-            <Button
-              colorPalette="brand"
-              onClick={() => setGranularity("hourly")}
-              variant={granularity === "hourly" ? "solid" : "outline"}
-            >
-              {translate("calendar.hourly")}
-            </Button>
-          </ButtonGroup>
+          <ButtonGroupToggle
+            onChange={setGranularity}
+            options={[
+              { label: translate("calendar.daily"), value: "daily" },
+              { label: translate("calendar.hourly"), value: "hourly" },
+            ]}
+            value={granularity}
+          />
 
-          <ButtonGroup attached size="sm" variant="outline">
-            <Button
-              colorPalette="brand"
-              onClick={() => setViewMode("total")}
-              variant={viewMode === "total" ? "solid" : "outline"}
-            >
-              {translate("calendar.totalRuns")}
-            </Button>
-            <Button
-              colorPalette="brand"
-              onClick={() => setViewMode("failed")}
-              variant={viewMode === "failed" ? "solid" : "outline"}
-            >
-              {translate("overview.buttons.failedRun_other")}
-            </Button>
-          </ButtonGroup>
+          <ButtonGroupToggle
+            onChange={setViewMode}
+            options={[
+              { label: translate("calendar.totalRuns"), value: "total" },
+              { label: translate("overview.buttons.failedRun_other"), value: "failed" },
+            ]}
+            value={viewMode}
+          />
         </HStack>
       </HStack>
 
@@ -222,6 +209,7 @@ export const Calendar = () => {
             bg="bg/80"
             borderRadius="md"
             bottom="0"
+            data-testid="calendar-loading-overlay"
             display="flex"
             justifyContent="center"
             left="0"
@@ -247,6 +235,7 @@ export const Calendar = () => {
           <>
             <DailyCalendarView
               data={data?.dag_runs ?? []}
+              data-testid="calendar-daily-view"
               scale={scale}
               selectedYear={selectedDate.year()}
               viewMode={viewMode}
