@@ -20,6 +20,7 @@ import json
 import logging
 import os
 import signal
+import socket
 import subprocess
 import time
 
@@ -46,6 +47,7 @@ from tests_common.test_utils.otel_utils import (
     assert_span_name_belongs_to_root_span,
     assert_span_not_in_children_spans,
     dump_airflow_metadata_db,
+    extract_metrics_from_output,
     extract_spans_from_output,
     get_parent_child_dict,
 )
@@ -54,6 +56,44 @@ from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, AIRFLOW_V
 log = logging.getLogger("integration.otel.test_otel")
 
 
+def wait_for_otel_collector(host: str, port: int, timeout: int = 120) -> None:
+    """
+    Wait for the OTel collector to be reachable before running tests.
+
+    This prevents flaky test failures caused by transient DNS resolution issues
+    (e.g., 'Temporary failure in name resolution' for breeze-otel-collector).
+
+    Note: If the collector is not reachable after timeout, logs a warning but
+    does not fail - allows tests to run and fail naturally if needed.
+    """
+    deadline = time.monotonic() + timeout
+    last_error = None
+    while time.monotonic() < deadline:
+        try:
+            # Test DNS resolution and TCP connectivity
+            with socket.create_connection((host, port), timeout=5):
+                pass
+            log.info("OTel collector at %s:%d is reachable.", host, port)
+            return
+        except (socket.gaierror, TimeoutError, OSError) as e:
+            last_error = e
+            log.debug(
+                "OTel collector at %s:%d not reachable: %s. Retrying...",
+                host,
+                port,
+                e,
+            )
+            time.sleep(2)
+    log.warning(
+        "OTel collector at %s:%d is not reachable after %ds. Last error: %s. "
+        "Tests will proceed but may fail if collector is required.",
+        host,
+        port,
+        timeout,
+        last_error,
+    )
+
+
 def unpause_trigger_dag_and_get_run_id(dag_id: str) -> str:
     unpause_command = ["airflow", "dags", "unpause", dag_id]
 
@@ -152,6 +192,52 @@ def check_ti_state_and_span_status(task_id: str, run_id: str, state: str, span_s
             )
 
 
+def check_legacy_metrics(output: str, dag: DAG, legacy_metrics_on: bool):
+    # Get a list of lines from the captured output.
+    output_lines = output.splitlines()
+
+    metrics_dict = extract_metrics_from_output(output_lines)
+
+    # Sample of metrics to check:
+    #   new:    airflow.dagrun.dependency-check
+    #   legacy: airflow.dagrun.dependency-check.otel_test_dag
+    #
+    #   new:    airflow.task.scheduled_duration
+    #   legacy: airflow.dag.otel_test_dag.task1.scheduled_duration
+    #   legacy: airflow.dag.otel_test_dag.task2.scheduled_duration
+    #
+    #   new:    airflow.dagrun.duration.success
+    #   legacy: airflow.dagrun.duration.success.otel_test_dag
+
+    legacy_metric_names = [
+        f"airflow.dagrun.dependency-check.{dag.dag_id}",
+        f"airflow.dagrun.duration.success.{dag.dag_id}",
+    ]
+
+    for task_id in dag.task_ids:
+        legacy_metric_names.append(f"airflow.dag.{dag.dag_id}.{task_id}.scheduled_duration")
+
+    new_metric_names = [
+        "airflow.dagrun.dependency-check",
+        "airflow.dagrun.duration.success",
+        "airflow.task.scheduled_duration",
+    ]
+
+    assert set(new_metric_names).issubset(metrics_dict.keys())
+
+    if legacy_metrics_on:
+        assert set(legacy_metric_names).issubset(metrics_dict.keys())
+
+
+def check_metrics_exist(output: str, metrics_to_check: list[str]):
+    # Get a list of lines from the captured output.
+    output_lines = output.splitlines()
+
+    metrics_dict = extract_metrics_from_output(output_lines)
+
+    assert set(metrics_to_check).issubset(metrics_dict.keys())
+
+
 def check_spans_with_continuance(output: str, dag: DAG, continuance_for_t1: bool = True):
     # Get a list of lines from the captured output.
     output_lines = output.splitlines()
@@ -609,11 +695,19 @@ class TestOtelIntegration:
 
     @classmethod
     def setup_class(cls):
+        otel_host = "breeze-otel-collector"
+        otel_port = 4318
+
+        # Wait for OTel collector to be reachable before running tests.
+        # This prevents flaky test failures caused by transient DNS resolution issues
+        # during scheduler handoff (see https://github.com/apache/airflow/issues/61070).
+        wait_for_otel_collector(otel_host, otel_port)
+
         os.environ["AIRFLOW__TRACES__OTEL_ON"] = "True"
-        os.environ["AIRFLOW__TRACES__OTEL_HOST"] = "breeze-otel-collector"
-        os.environ["AIRFLOW__TRACES__OTEL_PORT"] = "4318"
+        os.environ["OTEL_EXPORTER_OTLP_PROTOCOL"] = "http/protobuf"
+        os.environ["OTEL_EXPORTER_OTLP_TRACES_ENDPOINT"] = "http://breeze-otel-collector:4318/v1/traces"
         if cls.use_otel != "true":
-            os.environ["AIRFLOW__TRACES__OTEL_DEBUGGING_ON"] = "True"
+            os.environ["OTEL_TRACES_EXPORTER"] = "console"
 
         os.environ["AIRFLOW__SCHEDULER__STANDALONE_DAG_PROCESSOR"] = "False"
         os.environ["AIRFLOW__SCHEDULER__PROCESSOR_POLL_INTERVAL"] = "2"
@@ -727,6 +821,119 @@ class TestOtelIntegration:
         except Exception as ex:
             log.error("Could not delete leftover control file '%s', error: '%s'.", self.control_file, ex)
 
+    def dag_execution_for_testing_metrics(self, capfd):
+        # Metrics.
+        os.environ["AIRFLOW__METRICS__OTEL_ON"] = "True"
+        os.environ["OTEL_EXPORTER_OTLP_METRICS_ENDPOINT"] = "http://breeze-otel-collector:4318/v1/metrics"
+        os.environ["OTEL_METRIC_EXPORT_INTERVAL"] = "5000"
+
+        if self.use_otel != "true":
+            os.environ["OTEL_METRICS_EXPORTER"] = "console"
+
+        celery_worker_process = None
+        scheduler_process = None
+        apiserver_process = None
+        try:
+            # Start the processes here and not as fixtures or in a common setup,
+            # so that the test can capture their output.
+            celery_worker_process, scheduler_process, apiserver_process = self.start_worker_and_scheduler1()
+
+            dag_id = "otel_test_dag"
+
+            assert len(self.dags) > 0
+            dag = self.dags[dag_id]
+
+            assert dag is not None
+
+            run_id = unpause_trigger_dag_and_get_run_id(dag_id=dag_id)
+
+            # Skip the span_status check.
+            wait_for_dag_run_and_check_span_status(
+                dag_id=dag_id, run_id=run_id, max_wait_time=90, span_status=None
+            )
+
+            # The ti span_status is updated while processing the executor events,
+            # which is after the dag_run state has been updated.
+            time.sleep(10)
+
+            task_dict = dag.task_dict
+            task_dict_ids = task_dict.keys()
+
+            for task_id in task_dict_ids:
+                # Skip the span_status check.
+                check_ti_state_and_span_status(
+                    task_id=task_id, run_id=run_id, state=State.SUCCESS, span_status=None
+                )
+
+            print_ti_output_for_dag_run(dag_id=dag_id, run_id=run_id)
+        finally:
+            # Terminate the processes.
+            celery_worker_process.terminate()
+            celery_worker_process.wait()
+
+            celery_status = celery_worker_process.poll()
+            assert celery_status is not None, (
+                "The celery worker process status is None, which means that it hasn't terminated as expected."
+            )
+
+            scheduler_process.terminate()
+            scheduler_process.wait()
+
+            scheduler_status = scheduler_process.poll()
+            assert scheduler_status is not None, (
+                "The scheduler_1 process status is None, which means that it hasn't terminated as expected."
+            )
+
+            apiserver_process.terminate()
+            apiserver_process.wait()
+
+            apiserver_status = apiserver_process.poll()
+            assert apiserver_status is not None, (
+                "The apiserver process status is None, which means that it hasn't terminated as expected."
+            )
+
+        out, err = capfd.readouterr()
+        log.info("out-start --\n%s\n-- out-end", out)
+        log.info("err-start --\n%s\n-- err-end", err)
+
+        return out, dag
+
+    @pytest.mark.parametrize(
+        ("legacy_names_on_bool", "legacy_names_exported"),
+        [
+            pytest.param(True, True, id="export_legacy_names"),
+            pytest.param(False, False, id="dont_export_legacy_names"),
+        ],
+    )
+    def test_export_legacy_metric_names(
+        self, legacy_names_on_bool, legacy_names_exported, monkeypatch, celery_worker_env_vars, capfd, session
+    ):
+        assert isinstance(legacy_names_on_bool, bool)
+        os.environ["AIRFLOW__METRICS__LEGACY_NAMES_ON"] = str(legacy_names_on_bool)
+
+        out, dag = self.dag_execution_for_testing_metrics(capfd)
+
+        if self.use_otel != "true":
+            # Test the metrics from the output.
+            assert isinstance(legacy_names_exported, bool)
+            check_legacy_metrics(output=out, dag=dag, legacy_metrics_on=legacy_names_exported)
+
+    def test_export_metrics_during_process_shutdown(
+        self, monkeypatch, celery_worker_env_vars, capfd, session
+    ):
+        out, dag = self.dag_execution_for_testing_metrics(capfd)
+
+        if self.use_otel != "true":
+            # Test the metrics from the output.
+            metrics_to_check = [
+                "airflow.ti_successes",
+                "airflow.operator_successes",
+                "airflow.executor.running_tasks",
+                "airflow.executor.queued_tasks",
+                "airflow.executor.open_slots",
+            ]
+            check_metrics_exist(output=out, metrics_to_check=metrics_to_check)
+
     @pytest.mark.execution_timeout(90)
     def test_dag_execution_succeeds(self, monkeypatch, celery_worker_env_vars, capfd, session):
         """The same scheduler will start and finish the dag processing."""
