@@ -56,6 +56,7 @@ from airflow.dag_processing.processor import (
     DagFileProcessorProcess,
     ToDagProcessor,
     ToManager,
+    _execute_callbacks,
     _execute_dag_callbacks,
     _execute_email_callbacks,
     _execute_task_callbacks,
@@ -609,6 +610,41 @@ def test_parse_file_with_task_callbacks(spy_agency):
     assert called is True
 
 
+@conf_vars({("dag_processor", "dag_version_inflation_check_level"): "error"})
+def test_parse_file_static_check_with_error():
+    result = _parse_file(
+        DagFileParseRequest(
+            file=f"{TEST_DAG_FOLDER}/test_dag_version_inflation_check.py",
+            bundle_path=TEST_DAG_FOLDER,
+            bundle_name="testing",
+        ),
+        log=structlog.get_logger(),
+    )
+
+    assert result.serialized_dags == []
+    assert list(result.import_errors.keys()) == ["test_dag_version_inflation_check.py"]
+    assert result.warnings is None
+    assert "Don't use the variables as arguments" in next(iter(result.import_errors.values()))
+
+
+def test_parse_file_static_check_with_default_warning():
+    result = _parse_file(
+        DagFileParseRequest(
+            file=f"{TEST_DAG_FOLDER}/test_dag_version_inflation_check.py",
+            bundle_path=TEST_DAG_FOLDER,
+            bundle_name="testing",
+        ),
+        log=structlog.get_logger(),
+    )
+
+    assert len(result.serialized_dags) > 0
+    assert len(result.warnings) == len(result.serialized_dags)
+    assert all(
+        warning.get("dag_id") and warning.get("warning_type") and warning.get("message")
+        for warning in result.warnings
+    )
+
+
 def test_callback_processing_does_not_update_timestamps(session):
     """Callback processing should not update last_finish_time to prevent stale DAG detection."""
     stat = process_parse_results(
@@ -666,6 +702,34 @@ def test_import_error_updates_timestamps(session):
     assert stat.import_errors == 1
 
 
+class TestExecuteCallbacks:
+    def test_execute_callbacks_locks_bundle_version(self):
+        callbacks = [
+            DagCallbackRequest(
+                filepath="test.py",
+                dag_id="test_dag",
+                run_id="test_run",
+                bundle_name="testing",
+                bundle_version="some_commit_hash",
+                is_failure_callback=False,
+                msg=None,
+            )
+        ]
+        log = MagicMock(spec=FilteringBoundLogger)
+        dagbag = MagicMock(spec=DagBag)
+
+        with (
+            patch("airflow.dag_processing.processor.BundleVersionLock") as mock_lock,
+            patch("airflow.dag_processing.processor._execute_dag_callbacks") as mock_execute,
+        ):
+            _execute_callbacks(dagbag, callbacks, log)
+
+        mock_lock.assert_called_once_with(bundle_name="testing", bundle_version="some_commit_hash")
+        mock_lock.return_value.__enter__.assert_called_once()
+        mock_lock.return_value.__exit__.assert_called_once()
+        mock_execute.assert_called_once_with(dagbag, callbacks[0], log)
+
+
 class TestExecuteDagCallbacks:
     """Test the _execute_dag_callbacks function with context_from_server"""
 
