@@ -201,6 +201,7 @@ class TestClearTasks:
         """
         Test that DagRun state, start_date and last_scheduling_decision
         are not changed after clearing TI in an unfinished DagRun.
+        However, queued_at and clear_number should still be updated.
         """
         # Explicitly needs catchup as True as test is creating history runs
         with dag_maker(
@@ -222,6 +223,10 @@ class TestClearTasks:
         session = dag_maker.session
         session.flush()
 
+        # Store original values to verify they're updated
+        original_queued_at = dr.queued_at
+        original_clear_number = dr.clear_number
+
         # we use order_by(task_id) here because for the test DAG structure of ours
         # this is equivalent to topological sort. It would not work in general case
         # but it works for our case because we specifically constructed test DAGS
@@ -239,6 +244,11 @@ class TestClearTasks:
             assert dr.start_date
         assert dr.last_scheduling_decision == DEFAULT_DATE
 
+        # Verify queued_at and clear_number are updated even for running/queued dag runs
+        assert dr.queued_at is not None
+        assert dr.queued_at != original_queued_at
+        assert dr.clear_number == original_clear_number + 1
+
     @pytest.mark.parametrize(
         ("state", "last_scheduling"),
         [
