@@ -39,6 +39,7 @@ from airflow.providers.common.compat.sdk import (
     BaseHook,
     conf,
 )
+from airflow.providers.common.sql.hooks.lineage import send_sql_hook_lineage
 from airflow.providers.common.sql.hooks.sql import DbApiHook
 from airflow.security import utils
 from airflow.utils.helpers import as_flattened_list
@@ -332,6 +333,8 @@ class HiveCliHook(BaseHook):
             if sub_process.returncode:
                 raise AirflowException(stdout)
 
+            send_sql_hook_lineage(context=self, sql=hql)
+
             return stdout
 
     def test_hql(self, hql: str) -> None:
@@ -916,6 +919,7 @@ class HiveServer2Hook(DbApiHook):
 
             for statement in sql:
                 cur.execute(statement)
+                send_sql_hook_lineage(context=self, sql=statement, cur=cur, default_schema=schema)
                 # we only get results of statements that returns
                 lowered_statement = statement.lower().strip()
                 if lowered_statement.startswith(("select", "with", "show")) or (
@@ -1038,7 +1042,7 @@ class HiveServer2Hook(DbApiHook):
         try:
             import pandas as pd
         except ImportError as e:
-            from airflow.exceptions import AirflowOptionalProviderFeatureException
+            from airflow.providers.common.compat.sdk import AirflowOptionalProviderFeatureException
 
             raise AirflowOptionalProviderFeatureException(e)
 
@@ -1057,7 +1061,7 @@ class HiveServer2Hook(DbApiHook):
         try:
             import polars as pl
         except ImportError as e:
-            from airflow.exceptions import AirflowOptionalProviderFeatureException
+            from airflow.providers.common.compat.sdk import AirflowOptionalProviderFeatureException
 
             raise AirflowOptionalProviderFeatureException(e)
 
