@@ -22,6 +22,8 @@ import logging
 import os
 import re
 import shutil
+import tempfile
+import uuid
 from io import StringIO
 from pathlib import Path
 from unittest import mock
@@ -36,7 +38,10 @@ from airflow.providers.common.compat.sdk import conf
 from airflow.providers.elasticsearch.log.es_response import ElasticSearchResponse
 from airflow.providers.elasticsearch.log.es_task_handler import (
     VALID_ES_CONFIG_KEYS,
+    ElasticsearchRemoteLogIO,
     ElasticsearchTaskHandler,
+    _clean_date,
+    _render_log_id,
     get_es_kwargs_from_config,
     getattr_nested,
 )
@@ -54,10 +59,11 @@ from unit.elasticsearch.log.elasticmock.utilities import SearchFailedException
 ES_PROVIDER_YAML_FILE = AIRFLOW_PROVIDERS_ROOT_PATH / "elasticsearch" / "provider.yaml"
 
 
-def get_ti(dag_id, task_id, logical_date, create_task_instance):
+def get_ti(dag_id, task_id, run_id, logical_date, create_task_instance):
     ti = create_task_instance(
         dag_id=dag_id,
         task_id=task_id,
+        run_id=run_id,
         logical_date=logical_date,
         dagrun_state=DagRunState.RUNNING,
         state=TaskInstanceState.RUNNING,
@@ -70,9 +76,12 @@ def get_ti(dag_id, task_id, logical_date, create_task_instance):
 class TestElasticsearchTaskHandler:
     DAG_ID = "dag_for_testing_es_task_handler"
     TASK_ID = "task_for_testing_es_log_handler"
+    RUN_ID = "run_for_testing_es_log_handler"
+    MAP_INDEX = -1
+    TRY_NUM = 1
     LOGICAL_DATE = datetime(2016, 1, 1)
-    LOG_ID = f"{DAG_ID}-{TASK_ID}-2016-01-01T00:00:00+00:00-1"
-    JSON_LOG_ID = f"{DAG_ID}-{TASK_ID}-{ElasticsearchTaskHandler._clean_date(LOGICAL_DATE)}-1"
+    LOG_ID = f"{DAG_ID}-{TASK_ID}-{RUN_ID}-{MAP_INDEX}-{TRY_NUM}"
+    JSON_LOG_ID = f"{DAG_ID}-{TASK_ID}-{_clean_date(LOGICAL_DATE)}-1"
     FILENAME_TEMPLATE = "{try_number}.log"
 
     @pytest.fixture
@@ -88,6 +97,7 @@ class TestElasticsearchTaskHandler:
         yield get_ti(
             dag_id=self.DAG_ID,
             task_id=self.TASK_ID,
+            run_id=self.RUN_ID,
             logical_date=self.LOGICAL_DATE,
             create_task_instance=create_task_instance,
         )
@@ -128,21 +138,24 @@ class TestElasticsearchTaskHandler:
     def teardown_method(self):
         shutil.rmtree(self.local_log_location.split(os.path.sep)[0], ignore_errors=True)
 
-    def test_es_response(self):
-        sample_response = self.es.sample_log_response()
-        es_response = ElasticSearchResponse(self.es_task_handler, sample_response)
-        logs_by_host = self.es_task_handler._group_logs_by_host(es_response)
-
-        def concat_logs(lines):
-            log_range = -1 if lines[-1].message == self.es_task_handler.end_of_log_mark else None
-            return "\n".join(self.es_task_handler._format_msg(line) for line in lines[:log_range])
+    @pytest.mark.parametrize(
+        "sample_response",
+        [
+            pytest.param(lambda self: self.es.sample_airflow_2_log_response(), id="airflow_2"),
+            pytest.param(lambda self: self.es.sample_airflow_3_log_response(), id="airflow_3"),
+        ],
+    )
+    def test_es_response(self, sample_response):
+        response = sample_response(self)
+        es_response = ElasticSearchResponse(self.es_task_handler, response)
+        logs_by_host = self.es_task_handler.io._group_logs_by_host(es_response)
 
         for hosted_log in logs_by_host.values():
-            message = concat_logs(hosted_log)
+            message = self.es_task_handler.concat_logs(hosted_log)
 
         assert (
             message == "Dependencies all met for dep_context=non-requeueable"
-            " deps ti=<TaskInstance: example_bash_operator.run_after_loop owen_run_run [queued]>\n"
+            " deps ti=<TaskInstance: example_bash_operator.run_after_loop \n"
             "Starting attempt 1 of 1\nExecuting <Task(BashOperator): run_after_loop> "
             "on 2023-07-09 07:47:32+00:00"
         )
@@ -263,7 +276,7 @@ class TestElasticsearchTaskHandler:
     @pytest.mark.db_test
     def test_read_with_patterns_no_match(self, ti):
         ts = pendulum.now()
-        with mock.patch.object(self.es_task_handler, "index_patterns", new="test_other_*,test_another_*"):
+        with mock.patch.object(self.es_task_handler.io, "index_patterns", new="test_other_*,test_another_*"):
             logs, metadatas = self.es_task_handler.read(
                 ti, 1, {"offset": 0, "last_log_timestamp": str(ts), "end_of_log": False}
             )
@@ -280,14 +293,14 @@ class TestElasticsearchTaskHandler:
             metadata = metadatas[0]
 
         assert metadata["offset"] == "0"
-        assert not metadata["end_of_log"]
+        assert metadata["end_of_log"]
         # last_log_timestamp won't change if no log lines read.
         assert timezone.parse(metadata["last_log_timestamp"]) == ts
 
     @pytest.mark.db_test
     def test_read_with_missing_index(self, ti):
         ts = pendulum.now()
-        with mock.patch.object(self.es_task_handler, "index_patterns", new="nonexistent,test_*"):
+        with mock.patch.object(self.es_task_handler.io, "index_patterns", new="nonexistent,test_*"):
             with pytest.raises(elasticsearch.exceptions.NotFoundError, match=r"IndexMissingException.*"):
                 self.es_task_handler.read(
                     ti,
@@ -302,9 +315,11 @@ class TestElasticsearchTaskHandler:
         When the log actually isn't there to be found, we only want to wait for 5 seconds.
         In this case we expect to receive a message of the form 'Log {log_id} not found in elasticsearch ...'
         """
+        run_id = "wrong_run_id"
         ti = get_ti(
             self.DAG_ID,
             self.TASK_ID,
+            run_id,
             pendulum.instance(self.LOGICAL_DATE).add(days=1),  # so logs are not found
             create_task_instance=create_task_instance,
         )
@@ -320,7 +335,7 @@ class TestElasticsearchTaskHandler:
             else:
                 # we've "waited" less than 5 seconds so it should not be "end of log" and should be no log message
                 assert logs == []
-                assert metadatas["end_of_log"] is False
+                assert metadatas["end_of_log"] is True
             assert metadatas["offset"] == "0"
             assert timezone.parse(metadatas["last_log_timestamp"]) == ts
         else:
@@ -336,7 +351,7 @@ class TestElasticsearchTaskHandler:
                 # we've "waited" less than 5 seconds so it should not be "end of log" and should be no log message
                 assert len(logs[0]) == 0
                 assert logs == [[]]
-                assert metadatas[0]["end_of_log"] is False
+                assert metadatas[0]["end_of_log"] is True
             assert len(logs) == len(metadatas)
             assert metadatas[0]["offset"] == "0"
             assert timezone.parse(metadatas[0]["last_log_timestamp"]) == ts
@@ -432,7 +447,7 @@ class TestElasticsearchTaskHandler:
             metadata = metadatas[0]
 
         assert metadata["offset"] == "0"
-        assert not metadata["end_of_log"]
+        assert metadata["end_of_log"]
         # last_log_timestamp won't change if no log lines read.
         assert timezone.parse(metadata["last_log_timestamp"]) == ts
 
@@ -440,6 +455,7 @@ class TestElasticsearchTaskHandler:
     def test_read_with_empty_metadata(self, ti):
         ts = pendulum.now()
         logs, metadatas = self.es_task_handler.read(ti, 1, {})
+        print(f"metadatas: {metadatas}")
         if AIRFLOW_V_3_0_PLUS:
             logs = list(logs)
             assert logs[0].event == "::group::Log message source details"
@@ -455,7 +471,7 @@ class TestElasticsearchTaskHandler:
             assert self.test_message == logs[0][0][-1]
 
             metadata = metadatas[0]
-
+        print(f"metadatas: {metadatas}")
         assert not metadata["end_of_log"]
         # offset should be initialized to 0 if not provided.
         assert metadata["offset"] == "1"
@@ -477,7 +493,7 @@ class TestElasticsearchTaskHandler:
 
             metadata = metadatas[0]
 
-        assert not metadata["end_of_log"]
+        assert metadata["end_of_log"]
         # offset should be initialized to 0 if not provided.
         assert metadata["offset"] == "0"
         # last_log_timestamp will be initialized using log reading time
@@ -552,27 +568,22 @@ class TestElasticsearchTaskHandler:
 
     @pytest.mark.db_test
     def test_read_raises(self, ti):
-        with mock.patch.object(self.es_task_handler.log, "exception") as mock_exception:
-            with mock.patch.object(self.es_task_handler.client, "search") as mock_execute:
+        with mock.patch.object(self.es_task_handler.io.log, "exception") as mock_exception:
+            with mock.patch.object(self.es_task_handler.io.client, "search") as mock_execute:
                 mock_execute.side_effect = SearchFailedException("Failed to read")
-                logs, metadatas = self.es_task_handler.read(ti, 1)
+                log_sources, log_msgs = self.es_task_handler.io.read("", ti)
             assert mock_exception.call_count == 1
             args, kwargs = mock_exception.call_args
             assert "Could not read log with log_id:" in args[0]
 
         if AIRFLOW_V_3_0_PLUS:
-            assert logs == []
-
-            metadata = metadatas
+            assert log_sources == []
         else:
-            assert len(logs) == 1
-            assert len(logs) == len(metadatas)
-            assert logs == [[]]
+            assert len(log_sources) == 0
+            assert len(log_msgs) == 1
+            assert log_sources == []
 
-            metadata = metadatas[0]
-
-        assert metadata["offset"] == "0"
-        assert not metadata["end_of_log"]
+        assert "not found in Elasticsearch" in log_msgs[0]
 
     @pytest.mark.db_test
     def test_set_context(self, ti):
@@ -616,9 +627,7 @@ class TestElasticsearchTaskHandler:
             logs = list(logs)
             assert logs[2].event == self.test_message
         else:
-            assert (
-                logs[0][0][1] == "[2020-12-24 19:25:00,962] {taskinstance.py:851} INFO - some random stuff - "
-            )
+            assert logs[0][0][1] == self.test_message
 
     @pytest.mark.db_test
     def test_read_with_json_format_with_custom_offset_and_host_fields(self, ti):
@@ -634,7 +643,7 @@ class TestElasticsearchTaskHandler:
         self.body = {
             "message": self.test_message,
             "event": self.test_message,
-            "log_id": f"{self.DAG_ID}-{self.TASK_ID}-2016_01_01T00_00_00_000000-1",
+            "log_id": self.LOG_ID,
             "log": {"offset": 1},
             "host": {"name": "somehostname"},
             "asctime": "2020-12-24 19:25:00,962",
@@ -652,9 +661,7 @@ class TestElasticsearchTaskHandler:
             logs = list(logs)
             assert logs[2].event == self.test_message
         else:
-            assert (
-                logs[0][0][1] == "[2020-12-24 19:25:00,962] {taskinstance.py:851} INFO - some random stuff - "
-            )
+            assert logs[0][0][1] == self.test_message
 
     @pytest.mark.db_test
     def test_read_with_custom_offset_and_host_fields(self, ti):
@@ -753,13 +760,13 @@ class TestElasticsearchTaskHandler:
 
     @pytest.mark.db_test
     def test_render_log_id(self, ti):
-        assert self.es_task_handler._render_log_id(ti, 1) == self.LOG_ID
+        assert _render_log_id(self.es_task_handler.log_id_template, ti, 1) == self.LOG_ID
 
         self.es_task_handler.json_format = True
-        assert self.es_task_handler._render_log_id(ti, 1) == self.JSON_LOG_ID
+        assert _render_log_id(self.es_task_handler.log_id_template, ti, 1) == self.LOG_ID
 
     def test_clean_date(self):
-        clean_logical_date = self.es_task_handler._clean_date(datetime(2016, 7, 8, 9, 10, 11, 12))
+        clean_logical_date = _clean_date(datetime(2016, 7, 8, 9, 10, 11, 12))
         assert clean_logical_date == "2016_07_08T09_10_11_000012"
 
     @pytest.mark.db_test
@@ -770,7 +777,7 @@ class TestElasticsearchTaskHandler:
             (
                 True,
                 "localhost:5601/{log_id}",
-                "https://localhost:5601/" + quote(JSON_LOG_ID),
+                "https://localhost:5601/" + quote(LOG_ID),
             ),
             (
                 False,
@@ -867,8 +874,8 @@ class TestElasticsearchTaskHandler:
             mock_callable = Mock(return_value="callable_index_pattern")
             mock_import_string.return_value = mock_callable
 
-            self.es_task_handler.index_patterns_callable = "path.to.index_pattern_callable"
-            result = self.es_task_handler._get_index_patterns({})
+            self.es_task_handler.io.index_patterns_callable = "path.to.index_pattern_callable"
+            result = self.es_task_handler.io._get_index_patterns({})
 
             mock_import_string.assert_called_once_with("path.to.index_pattern_callable")
             mock_callable.assert_called_once_with({})
@@ -885,25 +892,6 @@ class TestElasticsearchTaskHandler:
             filename_template=None,
         )
 
-    @pytest.mark.db_test
-    def test_write_to_es(self, ti):
-        self.es_task_handler.write_to_es = True
-        self.es_task_handler.json_format = True
-        self.es_task_handler.write_stdout = False
-        self.es_task_handler.local_base = Path(os.getcwd()) / "local" / "log" / "location"
-        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
-        self.es_task_handler.formatter = formatter
-
-        self.es_task_handler.set_context(ti)
-        with patch(
-            "airflow.providers.elasticsearch.log.es_task_handler.ElasticsearchTaskHandler._write_to_es"
-        ) as mock_write_to_es:
-            mock_write = Mock(return_value=True)
-            mock_write_to_es.return_value = mock_write
-            self.es_task_handler._write_to_es = mock_write_to_es
-            self.es_task_handler.close()
-            mock_write_to_es.assert_called_once()
-
 
 def test_safe_attrgetter():
     class A: ...
@@ -963,3 +951,134 @@ def test_self_not_valid_arg():
     Test if self is not a valid argument.
     """
     assert "self" not in VALID_ES_CONFIG_KEYS
+
+
+@pytest.mark.db_test
+class TestElasticsearchRemoteLogIO:
+    DAG_ID = "dag_for_testing_es_log_handler"
+    TASK_ID = "task_for_testing_es_log_handler"
+    RUN_ID = "run_for_testing_es_log_handler"
+    LOGICAL_DATE = datetime(2016, 1, 1)
+    FILENAME_TEMPLATE = "{try_number}.log"
+
+    @pytest.fixture(autouse=True)
+    def setup_tests(self, ti):
+        self.elasticsearch_io = ElasticsearchRemoteLogIO(
+            write_to_es=True,
+            write_stdout=True,
+            delete_local_copy=True,
+            host="http://localhost:9200",
+            base_log_folder=Path(""),
+        )
+
+    @pytest.fixture
+    def tmp_json_file(self):
+        with tempfile.TemporaryDirectory() as tmpdir:
+            os.makedirs(tmpdir, exist_ok=True)
+
+            file_path = os.path.join(tmpdir, "1.log")
+            self.tmp_file = file_path
+
+            sample_logs = [
+                {"message": "start"},
+                {"message": "processing"},
+                {"message": "end"},
+            ]
+            with open(file_path, "w") as f:
+                for log in sample_logs:
+                    f.write(json.dumps(log) + "\n")
+
+            yield file_path
+
+            del self.tmp_file
+
+    @pytest.fixture
+    def ti(self, create_task_instance, create_log_template):
+        create_log_template(
+            self.FILENAME_TEMPLATE,
+            (
+                "{dag_id}-{task_id}-{logical_date}-{try_number}"
+                if AIRFLOW_V_3_0_PLUS
+                else "{dag_id}-{task_id}-{execution_date}-{try_number}"
+            ),
+        )
+        yield get_ti(
+            dag_id=self.DAG_ID,
+            task_id=self.TASK_ID,
+            run_id=self.RUN_ID,
+            logical_date=self.LOGICAL_DATE,
+            create_task_instance=create_task_instance,
+        )
+        clear_db_runs()
+        clear_db_dags()
+
+    @pytest.fixture
+    def unique_index(self):
+        """Generate a unique index name for each test."""
+        return f"airflow-logs-{uuid.uuid4()}"
+
+    @pytest.mark.integration("elasticsearch")
+    @pytest.mark.setup_timeout(300)
+    @pytest.mark.execution_timeout(300)
+    @patch(
+        "airflow.providers.elasticsearch.log.es_task_handler.TASK_LOG_FIELDS",
+        ["message"],
+    )
+    def test_read_write_to_es(self, tmp_json_file, ti, es_8_container_url):
+        self.elasticsearch_io.host = es_8_container_url
+        self.elasticsearch_io.client = elasticsearch.Elasticsearch(es_8_container_url).options(
+            request_timeout=120, retry_on_timeout=True, max_retries=5
+        )
+        self.elasticsearch_io.write_stdout = False
+        self.elasticsearch_io.upload(tmp_json_file, ti)
+        self.elasticsearch_io.client.indices.refresh(
+            index=self.elasticsearch_io.target_index, request_timeout=120
+        )
+        log_source_info, log_messages = self.elasticsearch_io.read("", ti)
+        assert log_source_info[0] == es_8_container_url
+        assert len(log_messages) == 3
+
+        expected_msg = ["start", "processing", "end"]
+        for msg, log_message in zip(expected_msg, log_messages):
+            print(f"msg: {msg}, log_message: {log_message}")
+            json_log = json.loads(log_message)
+            assert "message" in json_log
+            assert json_log["message"] == msg
+
+    def test_write_to_stdout(self, tmp_json_file, ti, capsys):
+        self.elasticsearch_io.write_to_es = False
+        self.elasticsearch_io.upload(tmp_json_file, ti)
+
+        captured = capsys.readouterr()
+        stdout_lines = captured.out.strip().splitlines()
+        log_entries = [json.loads(line) for line in stdout_lines]
+        assert log_entries[0]["message"] == "start"
+        assert log_entries[1]["message"] == "processing"
+        assert log_entries[2]["message"] == "end"
+
+    def test_invalid_task_log_file_path(self, ti):
+        with (
+            patch.object(self.elasticsearch_io, "_parse_raw_log") as mock_parse,
+            patch.object(self.elasticsearch_io, "_write_to_es") as mock_write,
+        ):
+            self.elasticsearch_io.upload(Path("/invalid/path"), ti)
+
+            mock_parse.assert_not_called()
+            mock_write.assert_not_called()
+
+    def test_raw_log_should_contain_log_id_and_offset(self, tmp_json_file, ti):
+        with open(self.tmp_file) as f:
+            raw_log = f.read()
+        json_log_lines = self.elasticsearch_io._parse_raw_log(raw_log, ti)
+        assert len(json_log_lines) == 3
+        for json_log_line in json_log_lines:
+            assert "log_id" in json_log_line
+            assert "offset" in json_log_line
+
+    @patch("elasticsearch.Elasticsearch.count", return_value={"count": 0})
+    def test_read_with_missing_log(self, mocked_count, ti):
+        log_source_info, log_messages = self.elasticsearch_io.read("", ti)
+        log_id = _render_log_id(self.elasticsearch_io.log_id_template, ti, ti.try_number)
+        assert log_source_info == []
+        assert f"*** Log {log_id} not found in Elasticsearch" in log_messages[0]
+        mocked_count.assert_called_once()
