@@ -232,7 +232,7 @@ class TestBigtableInstanceCreate:
 
 class TestBigtableInstanceUpdate:
     @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
-    def test_delete_execute(self, mock_hook):
+    def test_update_execute(self, mock_hook):
         op = BigtableUpdateInstanceOperator(
             project_id=PROJECT_ID,
             instance_id=INSTANCE_ID,
@@ -455,8 +455,8 @@ class TestBigtableClusterUpdate:
     @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
     def test_updating_cluster_that_does_not_exists(self, mock_hook):
         instance = mock_hook.return_value.get_instance.return_value = mock.Mock(Instance)
-        mock_hook.return_value.update_cluster.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Cluster not found.")
+        mock_hook.return_value.update_cluster.side_effect = AirflowException(
+            f"Dependency: cluster '{CLUSTER_ID}' does not exist for instance '{INSTANCE_ID}'."
         )
         op = BigtableUpdateClusterOperator(
             project_id=PROJECT_ID,
@@ -483,8 +483,8 @@ class TestBigtableClusterUpdate:
     @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
     def test_updating_cluster_that_does_not_exists_empty_project_id(self, mock_hook):
         instance = mock_hook.return_value.get_instance.return_value = mock.Mock(Instance)
-        mock_hook.return_value.update_cluster.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Cluster not found.")
+        mock_hook.return_value.update_cluster.side_effect = AirflowException(
+            f"Dependency: cluster '{CLUSTER_ID}' does not exist for instance '{INSTANCE_ID}'."
         )
         op = BigtableUpdateClusterOperator(
             instance_id=INSTANCE_ID,
@@ -584,7 +584,10 @@ class TestBigtableInstanceDelete:
         mock_hook.assert_not_called()
 
     @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
-    def test_deleting_instance_that_doesnt_exists(self, mock_hook):
+    def test_delete_instance_when_instance_missing_does_not_fail(self, mock_hook):
+        # Simulate hook handling missing instance gracefully
+        mock_hook.return_value.delete_instance.return_value = None
+
         op = BigtableDeleteInstanceOperator(
             project_id=PROJECT_ID,
             instance_id=INSTANCE_ID,
@@ -592,58 +595,9 @@ class TestBigtableInstanceDelete:
             gcp_conn_id=GCP_CONN_ID,
             impersonation_chain=IMPERSONATION_CHAIN,
         )
-        mock_hook.return_value.delete_instance.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Instance not found.")
-        )
+
         op.execute(None)
-        mock_hook.assert_called_once_with(
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-        mock_hook.return_value.delete_instance.assert_called_once_with(
-            project_id=PROJECT_ID, instance_id=INSTANCE_ID
-        )
 
-    @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
-    def test_deleting_instance_that_doesnt_exists_empty_project_id(self, mock_hook):
-        op = BigtableDeleteInstanceOperator(
-            instance_id=INSTANCE_ID,
-            task_id="id",
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-        mock_hook.return_value.delete_instance.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Instance not found.")
-        )
-        op.execute(None)
-        mock_hook.assert_called_once_with(
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-        mock_hook.return_value.delete_instance.assert_called_once_with(
-            project_id=None, instance_id=INSTANCE_ID
-        )
-
-    @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
-    def test_different_error_reraised(self, mock_hook):
-        op = BigtableDeleteInstanceOperator(
-            project_id=PROJECT_ID,
-            instance_id=INSTANCE_ID,
-            task_id="id",
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-        mock_hook.return_value.delete_instance.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.GoogleAPICallError("error")
-        )
-
-        with pytest.raises(google.api_core.exceptions.GoogleAPICallError):
-            op.execute(None)
-
-        mock_hook.assert_called_once_with(
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
         mock_hook.return_value.delete_instance.assert_called_once_with(
             project_id=PROJECT_ID, instance_id=INSTANCE_ID
         )
@@ -701,9 +655,6 @@ class TestBigtableTableDelete:
             impersonation_chain=IMPERSONATION_CHAIN,
         )
 
-        mock_hook.return_value.delete_table.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Table not found.")
-        )
         op.execute(None)
         mock_hook.assert_called_once_with(
             gcp_conn_id=GCP_CONN_ID,
@@ -723,9 +674,6 @@ class TestBigtableTableDelete:
             impersonation_chain=IMPERSONATION_CHAIN,
         )
 
-        mock_hook.return_value.delete_table.side_effect = mock.Mock(
-            side_effect=google.api_core.exceptions.NotFound("Table not found.")
-        )
         op.execute(None)
         mock_hook.assert_called_once_with(
             gcp_conn_id=GCP_CONN_ID,
@@ -735,28 +683,6 @@ class TestBigtableTableDelete:
             project_id=None, instance_id=INSTANCE_ID, table_id=TABLE_ID
         )
 
-    @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
-    def test_deleting_table_when_instance_doesnt_exists(self, mock_hook):
-        op = BigtableDeleteTableOperator(
-            project_id=PROJECT_ID,
-            instance_id=INSTANCE_ID,
-            table_id=TABLE_ID,
-            task_id="id",
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-
-        mock_hook.return_value.get_instance.return_value = None
-        with pytest.raises(AirflowException) as ctx:
-            op.execute(None)
-        err = ctx.value
-        assert str(err) == f"Dependency: instance '{INSTANCE_ID}' does not exist."
-        mock_hook.assert_called_once_with(
-            gcp_conn_id=GCP_CONN_ID,
-            impersonation_chain=IMPERSONATION_CHAIN,
-        )
-        mock_hook.return_value.delete_table.assert_not_called()
-
     @mock.patch("airflow.providers.google.cloud.operators.bigtable.BigtableHook")
     def test_different_error_reraised(self, mock_hook):
         op = BigtableDeleteTableOperator(
