@@ -22,6 +22,7 @@ from __future__ import annotations
 import copy
 import re
 import weakref
+from collections import deque
 from collections.abc import Generator, Iterator, Sequence
 from typing import TYPE_CHECKING, Any
 
@@ -36,6 +37,7 @@ from airflow.sdk.exceptions import (
 )
 
 if TYPE_CHECKING:
+    from airflow.sdk.api.datamodels._generated import DagAttributeTypes
     from airflow.sdk.bases.operator import BaseOperator
     from airflow.sdk.definitions._internal.abstractoperator import AbstractOperator
     from airflow.sdk.definitions._internal.expandinput import DictOfListsExpandInput, ListOfDictsExpandInput
@@ -43,7 +45,6 @@ if TYPE_CHECKING:
     from airflow.sdk.definitions.dag import DAG
     from airflow.sdk.definitions.edges import EdgeModifier
     from airflow.sdk.types import Operator
-    from airflow.serialization.enums import DagAttributeTypes
 
 
 def _default_parent_group() -> TaskGroup | None:
@@ -493,7 +494,7 @@ class TaskGroup(DAGNode):
 
     def serialize_for_task_group(self) -> tuple[DagAttributeTypes, Any]:
         """Serialize task group; required by DagNode."""
-        from airflow.serialization.enums import DagAttributeTypes
+        from airflow.sdk.api.datamodels._generated import DagAttributeTypes
         from airflow.serialization.serialized_objects import TaskGroupSerialization
 
         return (
@@ -586,10 +587,10 @@ class TaskGroup(DAGNode):
         """Return an iterator of the child tasks."""
         from airflow.sdk.definitions._internal.abstractoperator import AbstractOperator
 
-        groups_to_visit = [self]
+        groups_to_visit = deque([self])
 
         while groups_to_visit:
-            visiting = groups_to_visit.pop(0)
+            visiting = groups_to_visit.popleft()
 
             for child in visiting.children.values():
                 if isinstance(child, AbstractOperator):
